function SolveBigAllSpecies(PCVValue,HistCV)
    % This function runs the multi-scale agent-based B cell model.
    % Parameters for model simulations are inputed below.
    
    % run with:
    % >>SolveBigAllSpecies(asymmetricDivisionCV, FounderCellCV);
    % for example for 10% asymmetric division CV and 30% founder cell
    % kinetic rate parameter variance use:
    % >>SolveBigAllSpecies(0.1,0.3);
    % sample output with these parameters attached.
    % note founder cell CV is the CV for distributing the kinetic parameter
    % in the founder cells, the metabolite concentration resulting from
    % these parameters may not have the same CV.
    
    %maximum time for simulations
    maxTime=144*60;  % minutes
    
    N=1;  % starting cells
    
    replicates = 1;
    
    maxGens = 7;
    %start a parallel pool if one is not open
    if isempty(gcp('nocreate'))
        parpool('local',6);
    end

    %to be filled with parameters
    paramsToRun = {};
    paramNames = {};
    
     %%%%%%%%%%%%% Parameter DESCRIPTION %%%%%%%%%%%%%%
     % NFKB_CYCD_T NFKB_BCL2_T  NFKB_MYC_T    GROWTH_T  
     % RATE_CV CONC_CV PARTITION_CV VOL_CV IKK_CV  
     % GMSyn  GMDeg  MassGrowth MassDecay 
     % CYCDtSyn CYCDtDeg BCL2Syn BCL2tDeg 
     % MYCSyn MYCDeg mMycSyn mMycDeg L
     % IKK STIMULUS, Crel monomer,cRelFeedback,eps (Ribosome modifier)
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%      
    parameters= {'CD_T';'BCL_T';'MYC_T';'GROW_T';'RCV';'CCV';
        'PCV';'VCV';'IKKCV';'GMSyn';'GMDeg';'MGrow';'MDeg';'CDtSyn';'CDtDeg';
        'BCLtSyn';'BCLtDeg';'MYCSyn';'MYCDeg';'MYCtSyn';'MYCtDeg';'L';
        'IKK Fxn';'Crel?';'NFkBFbck';'EPS';'IKKTotal'};

    % Please adjust matlabpool to the number of cores available (maximum 12). 
    % If you are interested in running other conditions. Please uncomment the
    % parameters of interest below. 
    
   
     paramsToRun{end+1}=[40;40;40;40;         % CYCDT BCLT MYCT GROWT
        0.00;0.00;PCVValue;0.00;0.00 ;    % RCV CCV PCV VCV IKKCV
        0.002; 0.0002 ;0.004; 0.002;     % GMSyn GMDeg MGrow MDeg
        
        0.0030;0.002;13;0.007;            % CYCDtSyn CYCDtDeg BCL2tSyn BCL2tDeg 
        0.0040;0.0020;1;0.0231;1;         % MYCSyn MYCDeg mMycSyn mMycDeg L
        3;1;10;1;288.48;                     % IKK input (see intial_parameters8.m), cRel present?, NFkB induction modifier, translation modifier, IKK total (nM)
        HistCV;0;HistCV];                              %history RCV CCV
     parameterSetName=strcat('PCV',num2str(PCVValue),'-HCV',num2str(HistCV),'-',num2str(N),'Cell');
     paramNames{end+1}=parameterSetName;
 
 
    %run each parameter set
    for k = 1:numel(paramsToRun)
        
        %put conditions from parameters file in structure
        experiment = randomCondition(maxTime,paramsToRun{k});
        
        %store solutions in a Cell Array
        solutions ={};
        
        %replicates of the same conditions
        for rep = 1:replicates
            label = sprintf('%s_%d',paramNames{k},rep);
            %create an experiment label
            for i = 1:length(parameters)
                label= sprintf('%s %s=%2.2f',label,parameters{i},paramsToRun{k}(i));
            end
            fprintf(1,'%s\n',label);
            
            %run the model
            solutions{end+1}=runSolution(experiment,N,maxTime,maxGens);
            
            experiment_label=sprintf('%s_%d %s',paramNames{k},rep,datestr(now,'mmmm dd, yyyy HH-MM-SS-FFF'));
            
            %move the temporary folders and store them for future analysis
            movefile('unprocessedCells',strcat('unprocessedCells',experiment_label));
            movefile('processedCells',strcat('processedCells',experiment_label));
            movefile('Checkpoints',strcat('Checkpoints',experiment_label));
        end
        %save all outputs
        saveSolutions(solutions,N,maxTime,maxGens,paramNames{k},label);
    end
    %how long did that take?
    toc


    % matlabpool close;
    clear all;
    
    %clean up folders
    system('rm -rf unprocessedCells*');
    system('rm -rf processedCells*');
    system('rm -rf Checkpoints*');
    
end

function sol= runSolution(experiment, N, maxTime, maxGens)
    SEE = 0;
    
    fprintf(1,'\nChecking for saved progress\n');
    
    
    progressFiles=fullfile('Checkpoints','*.mat');
    solsToLoad=dir(progressFiles);
    %checkpointing in case the model crashes. This is not necessary any
    %more as the model run time has been greatly reduced. Validate before
    %attempting to continue a crashed simulation.
    if length(solsToLoad)>1
        error('too many solution files found');
        return;
    elseif ~isempty(solsToLoad)
        fileToLoad=fullfile('Checkpoints',solsToLoad(1).name);
        load(fileToLoad,'sol');
        fprintf(1,'\n1 Saved solution file found and loaded %s\n',fileToLoad);
    else
        fprintf(1,'\nNo saved solution files found, starting new analysis\n');
        sol = duplicate(experiment);    
        sol.counts=ones(ceil(maxTime/30),maxGens);
        sol.fullCounts = zeros(maxTime,maxGens); % total count as a function of gen and time
        sol.integralCounts = zeros(maxGens); % total number of generation cells
        sol.h24counts = zeros(maxGens); % total number of generation cells
        %average concentrations over time
        sol.raverages = zeros(100,144,maxGens);
        sol.qaverages = zeros(100,144,maxGens);
        sol.Tdivs= zeros(maxGens,ceil(maxTime/60)); % in terms of hours
        sol.Tdecides = zeros(maxGens,ceil(maxTime/60));
        sol.Tdies= zeros(maxGens,ceil(maxTime/60));
        sol.TdivTimes = cell(1);
        sol.TdieTimes = cell(1);
        sol.TdecideTimes = cell(1);
        sol.TdivMothers = cell(1);
        sol.TdieMothers = cell(1);
        sol.TdecideMothers = cell(1);    
        for i = 1:maxGens
            sol.TdivTimes{end+1}=cell(1);
            sol.TdieTimes{end+1}=cell(1);
            sol.TdecideTimes{end+1}=cell(1);
            sol.TdivMothers{end+1}=cell(1);
            sol.TdieMothers{end+1}=cell(1);
            sol.TdecideMothers{end+1}=cell(1);
        end

        sol.divbag = {};
        sol.diebag ={};
        sol.decidebag = {};
        
        sol.gs = zeros(maxGens); % fraction that grow to larger than v0+1.75 their starting size or to a size of 4 (both dividers and non-dividers)
        sol.fs = zeros(maxGens); % fraction that divides (assume that the rest will eventually die)
        sol.tp = zeros(maxGens); % fraction that grows and divides
        sol.fp = zeros(maxGens); % fraction that grows but dies
        sol.tn = zeros(maxGens); % fraction that doesn't grow and dies
        sol.fn = zeros(maxGens); % fraction that doesn't grow but divides
        sol.tp24 = zeros(maxGens); % fraction that grows and divides 24
        sol.fp24 = zeros(maxGens); % fraction that grows but dies 24
        sol.tn24 = zeros(maxGens); % fraction that doesn't grow and dies 24
        sol.fn24 = zeros(maxGens); % fraction that doesn't grow but divides 24
        
        %new sol parameters for individual NFkB species.
        %only storing every 30 minutes and N for number of cells this will
        %have to grow each time we add more cells.
        sol.RelA50n=zeros(ceil(maxTime/30)+1,N);
        sol.cRel50n=zeros(ceil(maxTime/30)+1,N);
        sol.RelA50Total=zeros(ceil(maxTime/30)+1,N);
        sol.cRel50Total=zeros(ceil(maxTime/30)+1,N);
        sol.RelAn=zeros(ceil(maxTime/30)+1,N);
        sol.cReln=zeros(ceil(maxTime/30)+1,N);
        sol.RelATotal=zeros(ceil(maxTime/30)+1,N);
        sol.cRelTotal=zeros(ceil(maxTime/30)+1,N);
        sol.BclXL=zeros(ceil(maxTime/30)+1,N);
        sol.CYCD=zeros(ceil(maxTime/30)+1,N);
        sol.Myc=zeros(ceil(maxTime/30)+1,N);
        sol.IkBa=zeros(ceil(maxTime/30)+1,N);
        sol.IkBb=zeros(ceil(maxTime/30)+1,N);
        sol.IkBe=zeros(ceil(maxTime/30)+1,N);
        sol.IkBd=zeros(ceil(maxTime/30)+1,N);
        sol.Mass=zeros(ceil(maxTime/30)+1,N);
        sol.cParp=zeros(ceil(maxTime/30)+1,N);
        sol.Parp=zeros(ceil(maxTime/30)+1,N);
        sol.c3Parp=zeros(ceil(maxTime/30)+1,N);
        sol.cdh1=zeros(ceil(maxTime/30)+1,N);

        sol.generationsOverTime=zeros(ceil(maxTime/3),maxGens);
                
        %Need to store lineage tree information for plotting modeling lineage trees.
        %CellID, motherID fateTime
        sol.lineageInfo=zeros(3,N);
  
        %save max min and range distribution for fitting to tracked data
        sol.minGenDistribution=zeros(N,1);
        sol.maxGenDistribution=zeros(N,1);
        sol.rangeDistribution=zeros(N,1);

    end
   
    vo = initial_parameters8(experiment);
    %total cell counts as a function of time
    
    %all of the cells currently waiting to be computed
    cells = cell(N,1);

    %Store how many total cells have been processed, this is used for the
    %plots of individual NFkB species to be stored correctly.
    totalCellNumberCurrentlyProcessed=1;
    
    TdivFits = zeros(maxGens,6); %lognormal fits: E[Tdiv]fit, s.d.[Tdiv]fit, E[Tdiv]min, E[Tdiv]max, s.d.[Tdiv]min, s.d.[Tdiv]max
    TdieFits = zeros(maxGens,6);
    optionsDiv  = odeset('AbsTol',1E-5,'RelTol',1E-3,'Events',@division);

    filesToProcess=fullfile('unprocessedCells','*.mat');
    cellsToProcess=dir(filesToProcess);
    
    %if we need to increase the size of the timecourse variables we're
    %storing let's do it here.
    if totalCellNumberCurrentlyProcessed+length(cellsToProcess)>size(sol.RelA50n,2)
        %we need to add more space for new cell timecourses to be stored.
        sol.RelA50n=[sol.RelA50n,zeros(size(sol.RelA50n,1),length(cellsToProcess))];
        sol.cRel50n=[sol.cRel50n,zeros(size(sol.cRel50n,1),length(cellsToProcess))];
        sol.RelA50Total=[sol.RelA50Total,zeros(size(sol.RelA50Total,1),length(cellsToProcess))];
        sol.cRel50Total=[sol.cRel50Total,zeros(size(sol.cRel50Total,1),length(cellsToProcess))];
        sol.RelAn=[sol.RelAn,zeros(size(sol.RelAn,1),length(cellsToProcess))];
        sol.cReln=[sol.RelAn,zeros(size(sol.cReln,1),length(cellsToProcess))];
        sol.RelATotal=[sol.RelATotal,zeros(size(sol.RelATotal,1),length(cellsToProcess))];
        sol.cRelTotal=[sol.cRelTotal,zeros(size(sol.cRelTotal,1),length(cellsToProcess))];
        sol.BclXL=[sol.BclXL,zeros(size(sol.BclXL,1),length(cellsToProcess))];
        sol.CYCD=[sol.CYCD,zeros(size(sol.CYCD,1),length(cellsToProcess))];
        sol.Myc=[sol.Myc,zeros(size(sol.Myc,1),length(cellsToProcess))];
        sol.IkBa=[sol.IkBa,zeros(size(sol.IkBa,1),length(cellsToProcess))];
        sol.IkBb=[sol.IkBb,zeros(size(sol.IkBb,1),length(cellsToProcess))];
        sol.IkBe=[sol.IkBe,zeros(size(sol.IkBe,1),length(cellsToProcess))];
        sol.IkBd=[sol.IkBd,zeros(size(sol.IkBd,1),length(cellsToProcess))];
        sol.Mass=[sol.Mass,zeros(size(sol.Mass,1),length(cellsToProcess))];
        sol.cParp=[sol.cParp,zeros(size(sol.cParp,1),length(cellsToProcess))];
        sol.Parp=[sol.Parp,zeros(size(sol.Parp,1),length(cellsToProcess))];
        sol.c3Parp=[sol.c3Parp,zeros(size(sol.c3Parp,1),length(cellsToProcess))];
        sol.cdh1=[sol.cdh1,zeros(size(sol.cdh1,1),length(cellsToProcess))];
        sol.lineageInfo=[sol.lineageInfo,zeros(size(sol.lineageInfo,1),length(cellsToProcess))];
    end
    
    if ~length(cellsToProcess)>0
    
        fprintf(1,'Initializing: %d cells\n',N);
        
        parfor i=1:N
        %uncomment to run without parallel toolbox
        %for i=1:N  
            dividing    = true;    % True if at equilibrium
            
            %%%%% RUN TO INITIALIZE THE PERSISTENT VARS %%%%%%%%%%
            v = initial_parameters8(experiment);
            
            %stochastic sampling of cell history.
            v= getRandomCell(v);
            
            nfkbDivDeath_ode_cRel([],[],[],v);

            v.SPECIES{1}(62) = 0;
            v.k2(77)= 1; 
            %%%%% RUN TO ACHIEVE STEADY STATE WITH BASAL SIGNALING %%%%%%%%%%%%%%
            dcount = 0;
            while dividing
                [t,x,te,ye,ie]=ode15s(@(t,x) nfkbDivDeath_ode_cRel(t,x,[],v), [0 24*60], v.SPECIES{1},optionsDiv);
                if ie ==2
                    %division occurred
                    x(end,(numel(v.SPECIES{1})-v.CELL_CYCLE+15))=x(end,(numel(v.SPECIES{1})-v.CELL_CYCLE+15))/2;
                    x(end,(numel(v.SPECIES{1})-v.CELL_CYCLE+13))=x(end,(numel(v.SPECIES{1})-v.CELL_CYCLE+13))/2;
                    x(end,126)=1; % avoid jumping back with cdh1
                    dcount=dcount+1;
                    if dcount > 1
                        fprintf(1,'Extra division with tdiv=%f\n',t(end));
                    end
                else
                    dividing = false;
                end
                v.SPECIES{1} = x(end,:);
                v.OUTPUT1 = [v.OUTPUT1;x];
            end
            v.SPECIES{1} = v.OUTPUT1(end,:)';
            v = getRandomCell(v);
            if experiment.VOL_CV > 0
                factor = max(0,normrnd(1,experiment.VOL_CV));
                v.SPECIES{1}(134)=v.SPECIES{1}(134)*factor;
                v.SPECIES{1}(132) =v.SPECIES{1}(132)*factor;
            end
            v.SPECIES{1}(62) = experiment.L;
            v.k2(77) = experiment.EPS;
            cells{i}.v=v;
            cells{i}.v.PHASE = 2;
            cells{i}.x = cells{i}.v.SPECIES{1};
            cells{i}.birthday=0;  % start at 0 hours
            cells{i}.gen = 1;  % undivided
            cells{i}.id =i;
            cells{i}.parent =0;
            cells{i}.maxgen=maxGens;
            cells{i}.founder=i;
        end
            fprintf('\nSaving cells to cells.out and processing initial cells\n');
            mkdir('unprocessedCells');
            mkdir('processedCells');
            save('-v7.3','unprocessedCells/cells.mat','cells');
            steadyStateMatrix=zeros(length(cells{1}.v.SPECIES{1}),N);
        for i=1:N
            steadyStateMatrix(:,i)=cells{i}.v.SPECIES{1};
        end
            experiment_label=sprintf('%s_%d %s',datestr(now,'mmmm dd, yyyy HH-MM-SS-FFF'));
            fid = fopen(strcat(experiment_label,'PreStimulusMetabolites.CSV'),'w');
            fprintf(fid, '%s,',cells{1}.v.SPECIES{2}{1:end});
            fprintf(fid, '\n');
            fclose(fid);
            dlmwrite(strcat(experiment_label,'PreStimulusMetabolites.CSV'),steadyStateMatrix','-append');
                    
    end
    
    cell_id=N+1;
    mkdir('Checkpoints');

    %process all the cells in the unprocessedCells folder until everything
    %is processed
    %newly divided cells will be added to unprocessed cells folder so once
    %this folder is empty the simulation is complete.
    while ~isempty(dir(fullfile('unprocessedCells','*.mat')))
        
        %let's load all files in the unprocessedCells folder and process
        %each one in turn
        filesToProcess=fullfile('unprocessedCells','*.mat');
        cellsToProcess=dir(filesToProcess);
            
        %if we need to increase the size of the timecourse variables we're
        %storing let's do it here.
        if totalCellNumberCurrentlyProcessed+length(cellsToProcess)>size(sol.RelA50n,2)
            %we need to add more space for new cell timecourses to
            %be stored.
            size(sol.RelA50n);
            ceil(maxTime/30);
            length(cellsToProcess);
            sol.RelA50n=[sol.RelA50n,zeros(size(sol.RelA50n,1),length(cellsToProcess))];
            sol.cRel50n=[sol.cRel50n,zeros(size(sol.cRel50n,1),length(cellsToProcess))];
            sol.RelA50Total=[sol.RelA50Total,zeros(size(sol.RelA50Total,1),length(cellsToProcess))];
            sol.cRel50Total=[sol.cRel50Total,zeros(size(sol.cRel50Total,1),length(cellsToProcess))];
            sol.RelAn=[sol.RelAn,zeros(size(sol.RelAn,1),length(cellsToProcess))];
            sol.cReln=[sol.RelAn,zeros(size(sol.cReln,1),length(cellsToProcess))];
            sol.RelATotal=[sol.RelATotal,zeros(size(sol.RelATotal,1),length(cellsToProcess))];
            sol.cRelTotal=[sol.cRelTotal,zeros(size(sol.cRelTotal,1),length(cellsToProcess))];
            sol.BclXL=[sol.BclXL,zeros(size(sol.BclXL,1),length(cellsToProcess))];
            sol.CYCD=[sol.CYCD,zeros(size(sol.CYCD,1),length(cellsToProcess))];
            sol.Myc=[sol.Myc,zeros(size(sol.Myc,1),length(cellsToProcess))];
            sol.IkBa=[sol.IkBa,zeros(size(sol.IkBa,1),length(cellsToProcess))];
            sol.IkBb=[sol.IkBb,zeros(size(sol.IkBb,1),length(cellsToProcess))];
            sol.IkBe=[sol.IkBe,zeros(size(sol.IkBe,1),length(cellsToProcess))];
            sol.IkBd=[sol.IkBd,zeros(size(sol.IkBd,1),length(cellsToProcess))];
            sol.Mass=[sol.Mass,zeros(size(sol.Mass,1),length(cellsToProcess))];
            sol.lineageInfo=[sol.lineageInfo,zeros(size(sol.lineageInfo,1),length(cellsToProcess))];
            sol.cParp=[sol.cParp,zeros(size(sol.cParp,1),length(cellsToProcess))];
            sol.Parp=[sol.Parp,zeros(size(sol.Parp,1),length(cellsToProcess))];
            sol.c3Parp=[sol.c3Parp,zeros(size(sol.c3Parp,1),length(cellsToProcess))];
            sol.cdh1=[sol.cdh1,zeros(size(sol.c3Parp,1),length(cellsToProcess))];
        end
        
        for k=1:length(cellsToProcess)
            load(fullfile('unprocessedCells',cellsToProcess(k).name));
            
            if length(cells)==0
                fprintf('\nEmpty saved cells found ignoring\n');
                mkdir('EmptyCells');
                fprintf(1,'\n Moving file to empty folder: %s\n',fullfile('EmptyCells',cellsToProcess(k).name));
                movefile(fullfile('unprocessedCells',cellsToProcess(k).name),fullfile('EmptyCells',cellsToProcess(k).name));
                continue;
            end
            stridelength=ceil(length(cells)/100);
            fprintf(1,'\nProcessing %d cells\n',length(cells));
            for stride=1:stridelength
                celllength = min(100,length(cells)-(stride-1)*100);
                
                parfor temporary=((stride-1)*100+1):((stride-1)*100)+celllength 
                %uncomment to run without parallel toolbox
                %for temporary=((stride-1)*100+1):((stride-1)*100)+celllength                   
                    cells{temporary}.v.SIM_TIME=max(1,(maxTime)-cells{temporary}.birthday);
                    cells{temporary}.v.START_TIME = cells{temporary}.birthday;
                    if cells{temporary}.gen == maxGens;
                        cells{temporary}.v=run_cell(cells{temporary}.v,true);
                    else
                        cells{temporary}.v=run_cell(cells{temporary}.v,false);
                    end
                    %placeholder times to be occupied by values, the
                    %minimum of which will determine cell fate
                    tdiv = 999999;
                    tdie = 999999;
                    tdecide=999999;
                    
                    if cells{temporary}.v.EVENT==1    % the cell must have died
                        tdie=cells{temporary}.v.SIM_TIME;    
                    elseif cells{temporary}.v.EVENT==2 %the cell must have divicded
                        tdiv=cells{temporary}.v.SIM_TIME;
                    else
                        %fprintf('something went wrong, cell didnt experience either fate\n');
                    end
                    
                    cells{temporary}.grew=0;
                    %check whether cell grew
                    if numel(cells{temporary}.v.OUTPUT2) > 0 && (cells{temporary}.v.OUTPUT2(end,134) > cells{temporary}.v.OUTPUT2(1,134)+2 || cells{temporary}.v.OUTPUT2(end,134) > 4.7)
                        tdecide=find(cells{temporary}.v.OUTPUT2(:,134)>cells{temporary}.v.OUTPUT2(1,134)*1.05,1,'first');
                        cells{temporary}.grew = 1;
                    end
                    
                    cells{temporary}.x = cells{temporary}.v.OUTPUT2(end,:);   % save the last concentrations
                    
                    %uses A LOT of memory
                    cells{temporary}.xc = cells{temporary}.v.OUTPUT2;   % save the entire trajectory?
                    cells{temporary}.tdiv = tdiv;
                    cells{temporary}.tdie = tdie;
                    cells{temporary}.tdecide= tdecide;
                end
            end
            
            %new cells are stored in these placeholder values
            cells2={};    
            cell_count = 0;
            real_count = 0;
            a_mass = 0;
            a_cparp=0;
            a_CD=0;
            a_GM=0;
            a_cRel=0;
            a_RelA=0;
            a_myc=0;
            a_bcl2=0;
            numdivs = 0;
            numdeaths = 0;
            numdecides = 0;
            numRelAPositive = 0;
            numcRelPositive = 0;
            sol.deaths = [];
            sol.divs = [];
            sol.decides = [];
            dividers = 0;
            nondividers = 0;
            Cgen = 0;        
        
            while ~isempty(cells)
                theCell = cells{1};
                a_mass=a_mass+theCell.x(134);
                a_cparp = a_cparp+theCell.x(84);
                a_CD = a_CD+theCell.x(122);
                a_GM = a_GM+theCell.x(132);
                a_RelA = a_RelA+theCell.x(33);
                a_cRel = a_cRel+theCell.x(53);
                a_myc=a_myc+theCell.x(143);
                a_bcl2=a_bcl2+theCell.x(94);
                if max(theCell.v.OUTPUT2(:,33)) > sol.NFKB_CYCD_T/2
                    numRelAPositive = numRelAPositive+1;
                end
                if max(theCell.v.OUTPUT2(:,53)) > sol.NFKB_CYCD_T/2
                    numcRelPositive = numcRelPositive+1;
                end
                
                if SEE == 1;
                    figure(1);
                    subplot(3,3,theCell.gen);
                    hold on;
                    %plot(theCell.xc(:,134));
                    %figure(2);
                    %subplot(3,3,theCell.gen);
                    %hold on;
                    %plot(theCell.xc(:,53));
                    %figure(3);
                    %subplot(3,3,theCell.gen);
                    %hold on;
                    %plot(theCell.xc(:,143));
                end
                
                real_count=real_count+1;

                sol.integralCounts(theCell.gen)=sol.integralCounts(theCell.gen)+1;
                bindex=ceil(theCell.birthday); % birth index
                index = ceil(theCell.v.SIM_TIME);    % length of the time course
                Cgen = theCell.gen;
                for i=((bindex)+1):30:index+bindex % for each time point in the timecourse
                    if i-bindex > 0 && i <= maxTime  
                        sol.counts(ceil(i/30),theCell.gen)=sol.counts(ceil(i/30),theCell.gen)+1; % add the counts to the summary
                    end
                end
                for i=((bindex)+1):1:(index+bindex) % for each time point in the timecourse
                    if i-bindex > 0 && i <= maxTime  
                        sol.fullCounts(i,theCell.gen)=sol.fullCounts(i,theCell.gen)+1; % add the counts to the summary
                    end
                end
                if theCell.tdecide <= maxTime;
                    sol.Tdecides(theCell.gen,ceil(theCell.tdecide/60))=sol.Tdecides(theCell.gen,ceil(theCell.tdecide/60))+1;
                    sol.decides(end+1)=ceil(theCell.tdecide/60);
                    sol.gs(theCell.gen)=sol.gs(theCell.gen)+1;
                    numdecides=numdecides+1;
                end
                %adding the cells to generationsOverTime
                startPosition=ceil(bindex/3)+1;
                endPosition=(startPosition+ceil(index/3));
                if startPosition<1
                    startPosition=1;
                end
                if endPosition>length(sol.generationsOverTime)
                    endPosition=length(length(sol.generationsOverTime));
                end
                onesToAdd=ones(endPosition-startPosition,1);
                
                sol.generationsOverTime(startPosition:(endPosition-1),Cgen)=sol.generationsOverTime(startPosition:(endPosition-1),Cgen)+onesToAdd;

                if (theCell.tdie <= theCell.tdiv)
                    fprintf('cell died at gen:%d,  tdie:%d, tdiv:%d, cParp:%.2f, cdh1:%.2f\n',theCell.gen,theCell.tdie,theCell.tdiv,theCell.v.OUTPUT2(end,84),theCell.v.OUTPUT2(end,126));
                    % cell died
                    %if the cell died beyond the last time set the death
                    %time to the last time.
                    if theCell.tdie+1+bindex > maxTime
                   %     fprintf('cell tdie+1+binde >maxTime, tdiv: %d, tdie %d, tdecide %d,bindex: %d, maxTime %d\n',theCell.tdiv,theCell.tdie,theCell.tdecide,bindex,maxTime);
                        theCell.tdie=(maxTime-bindex);
                    %    fprintf('new tDie: %d\n',theCell.tdie);
                    end
                    
                    %fprintf(1,'A cell died at time %f it was %f minutes old. Total cells %d\n',theCell.tdie+theCell.birthday,theCell.tdie,length(cells)+length(cells2));
                    numdeaths= numdeaths+1;
                    sol.Tdies(theCell.gen,ceil(theCell.tdie/60))=sol.Tdies(theCell.gen,ceil(theCell.tdie/60))+1;
                    sol.deaths(end+1)=ceil(theCell.tdie/60);
                    %The cell grew (this never happens but check just in case)
                    if theCell.grew
                        sol.fp(theCell.gen) = sol.fp(theCell.gen)+1;
                        if (theCell.tdie < 36*60 &&theCell.tdie > 12*60) || (theCell.gen == 1 && (theCell.tdie < 48*60 &&theCell.tdie > 24*60))
                            sol.fp24(theCell.gen) = sol.fp24(theCell.gen) +1;
                            sol.h24counts(theCell.gen) = sol.h24counts(theCell.gen) +1;
                        end
                        sol.TdecideTimes{theCell.gen}{end+1}=theCell.tdecide;
                        sol.TdecideMothers{theCell.gen}{end+1}=theCell.parent;
                    else
                        sol.tn(theCell.gen) = sol.tn(theCell.gen)+1;
                        if (theCell.tdie < 36*60 &&theCell.tdie > 12*60) || (theCell.gen == 1 && (theCell.tdie < 48*60 &&theCell.tdie > 24*60))
                            sol.tn24(theCell.gen) = sol.tn24(theCell.gen) +1;
                            sol.h24counts(theCell.gen) = sol.h24counts(theCell.gen) +1;
                        end
                    end
                    
                    for s=1:144
                        if size(theCell.v.OUTPUT2,1) > 100
                            for i = 1:1:100
                                sol.qaverages(i,s,theCell.gen)=sol.qaverages(i,s,theCell.gen)+theCell.v.OUTPUT2(ceil((size(theCell.v.OUTPUT2,1)*i)/100),s);
                            end
                        else
                            for i = 1:1:100
                                j=ceil((i*size(theCell.v.OUTPUT2,1))/100.0);
                                sol.qaverages(i,s,theCell.gen)=sol.qaverages(i,s,theCell.gen)+theCell.v.OUTPUT2(j,s);
                            end
                        end
                    end
                    sol.TdieTimes{theCell.gen}{end+1}=theCell.tdie;
                    sol.TdieMothers{theCell.gen}{end+1}=theCell.parent;
                    nondividers=nondividers+1;
                    cells(1)=[];
                    
                    %some species numbers to keep track of which species is which later in
                    %the program.
                    %NOTE THIS IS DEFINED IN TWO PLACES
                    RelA50nIndex=33;
                    cRel50nIndex=53;
                    RelA50TotalIndex=[32:41];
                    cRel50TotalIndex=[52:61];
                    RelAnIndex=[15,23:2:41];
                    cRelnIndex=[21,53:2:61];
                    RelATotalIndex=[14,15,22:41];
                    cRelTotalIndex=[20,21,52:61];
                    BclXLIndex=[61+33];
                    CYCDIndex=[61+58+3];
                    MycIndex=[61+58+24];
                    IkBaIndex=[1];
                    IkBbIndex=[4];
                    IkBeIndex=[7];
                    IkBdIndex=[10];
                    MassIndex=[61+28+15];
                    cParpIndex=[84];
                    ParpIndex=[82];
                    c3ParpIndex=[83];
                    cdh1Index=[126];
                    %cell died need to save it's trajectory into the Matrix
                    %of all metabolites.

                    if totalCellNumberCurrentlyProcessed+length(cellsToProcess)>size(sol.RelA50n,2)
                        %we need to add more space for new cell timecourses to
                        %be stored.
                        size(sol.RelA50n);
                        ceil(maxTime/30);
                            length(cellsToProcess);
                        sol.RelA50n=[sol.RelA50n,zeros(size(sol.RelA50n,1),length(cellsToProcess))];
                        sol.cRel50n=[sol.cRel50n,zeros(size(sol.cRel50n,1),length(cellsToProcess))];
                        sol.RelA50Total=[sol.RelA50Total,zeros(size(sol.RelA50Total,1),length(cellsToProcess))];
                        sol.cRel50Total=[sol.cRel50Total,zeros(size(sol.cRel50Total,1),length(cellsToProcess))];
                        sol.RelAn=[sol.RelAn,zeros(size(sol.RelAn,1),length(cellsToProcess))];
                        sol.cReln=[sol.RelAn,zeros(size(sol.cReln,1),length(cellsToProcess))];
                        sol.RelATotal=[sol.RelATotal,zeros(size(sol.RelATotal,1),length(cellsToProcess))];
                        sol.cRelTotal=[sol.cRelTotal,zeros(size(sol.cRelTotal,1),length(cellsToProcess))];
                        sol.BclXL=[sol.BclXL,zeros(size(sol.BclXL,1),length(cellsToProcess))];
                        sol.CYCD=[sol.CYCD,zeros(size(sol.CYCD,1),length(cellsToProcess))];
                        sol.Myc=[sol.Myc,zeros(size(sol.Myc,1),length(cellsToProcess))];
                        sol.IkBa=[sol.IkBa,zeros(size(sol.IkBa,1),length(cellsToProcess))];
                        sol.IkBb=[sol.IkBb,zeros(size(sol.IkBb,1),length(cellsToProcess))];
                        sol.IkBe=[sol.IkBe,zeros(size(sol.IkBe,1),length(cellsToProcess))];
                        sol.IkBd=[sol.IkBd,zeros(size(sol.IkBd,1),length(cellsToProcess))];
                        sol.Mass=[sol.Mass,zeros(size(sol.Mass,1),length(cellsToProcess))];
                        sol.lineageInfo=[sol.lineageInfo,zeros(size(sol.lineageInfo,1),length(cellsToProcess))];
                        sol.cParp=[sol.cParp,zeros(size(sol.cParp,1),length(cellsToProcess))];
                        sol.Parp=[sol.Parp,zeros(size(sol.Parp,1),length(cellsToProcess))];
                        sol.cdh1=[sol.cdh1,zeros(size(sol.c3Parp,1),length(cellsToProcess))];

                    end
        
                    birthday=ceil(theCell.birthday/30);
                    birthday=birthday+1;
                    deathTime=size(sum(theCell.v.OUTPUT2(1:30:end,RelA50nIndex),2),1)+birthday;
                    if size(sum(theCell.v.OUTPUT2(1:30:end,RelA50nIndex),2),1)+1==size(birthday:deathTime,2)
                        fprintf('adjusting death time from: %d to %d\n',deathTime,deathTime-1);
                        deathTime=deathTime-1;
                    end
                    try
                        sol.RelA50n(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,RelA50nIndex),2);
                    catch ME
                        try
                            %if the simulation does not error then this
                            %code didn't run. Should be ignored now.
                            fprintf('error, dimensions wrong. theCell.tdie: %d, theCell.birthday: %d\n',theCell.tdie,theCell.birthday);
                            deathTime=deathTime+1;
                            fprintf('Increased death Time deathTime: %d\n',deathTime);
                            sol.RelA50n(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,RelA50nIndex),2);
                        catch ME2
                            rethrow(ME);
                        end
                        
                        rethrow(ME);
                    end
                    
                    %store max and min gen per lineage to keep track of
                    %range
                    changed=false;
                    if(theCell.gen>sol.maxGenDistribution(theCell.founder))
                        sol.maxGenDistribution(theCell.founder)=theCell.gen;
                        changed=true;
                    end
                    if((sol.minGenDistribution(theCell.founder)==0)||theCell.gen<sol.minGenDistribution(theCell.founder))
                        sol.minGenDistribution(theCell.founder)=theCell.gen;
                        changed=true;
                    end
                    if(changed)
                         sol.rangeDistribution(theCell.founder)=sol.maxGenDistribution(theCell.founder)-sol.minGenDistribution(theCell.founder);
                    end
                    
                    sol.RelA50n(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.cRel50n(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,cRel50nIndex),2);
                    sol.cRel50n(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.RelA50Total(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,RelA50TotalIndex),2);
                    sol.RelA50Total(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.cRel50Total(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,cRel50TotalIndex),2);
                    sol.cRel50Total(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.RelAn(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,RelAnIndex),2);
                    sol.RelAn(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.cReln(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,cRelnIndex),2);
                    sol.cReln(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.RelATotal(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,RelATotalIndex),2);
                    sol.RelATotal(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.cRelTotal(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,cRelTotalIndex),2);
                    sol.cRelTotal(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.BclXL(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,BclXLIndex),2);
                    sol.BclXL(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.CYCD(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,CYCDIndex),2);
                    sol.CYCD(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.Myc(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,MycIndex),2);
                    sol.Myc(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.IkBa(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,IkBaIndex),2);
                    sol.IkBa(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.IkBb(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,IkBbIndex),2);
                    sol.IkBb(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.IkBe(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,IkBeIndex),2);
                    sol.IkBe(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.IkBd(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,IkBdIndex),2);
                    sol.IkBd(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.Mass(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,MassIndex),2);                    
                    sol.Mass(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    
                    sol.cParp(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,cParpIndex),2);
                    sol.cParp(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.Parp(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,ParpIndex),2);
                    sol.Parp(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.c3Parp(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,c3ParpIndex),2);
                    sol.c3Parp(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.cdh1(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,cdh1Index),2);
                    sol.cdh1(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.lineageInfo(1,totalCellNumberCurrentlyProcessed)= theCell.id;
                    sol.lineageInfo(2,totalCellNumberCurrentlyProcessed)=theCell.parent;
                    sol.lineageInfo(3,totalCellNumberCurrentlyProcessed)=theCell.birthday+theCell.tdie;
                    totalCellNumberCurrentlyProcessed=totalCellNumberCurrentlyProcessed+1;

                else
                    if theCell.tdiv+1+bindex>maxTime
                    %    fprintf('cell tDiv+1+bindex >maxTime, tdiv: %d, tdie %d, tdecide %d,bindex: %d, maxTime %d\n',theCell.tdiv,theCell.tdie,theCell.tdecide,bindex,maxTime);
                        theCell.tdiv=maxTime-bindex;    
                    %    fprintf('new tDiv: %d\n',theCell.tdiv);

                    end
                    
                    fprintf('cell divided at gen:%d,  tdie:%d, tdiv:%d, cParp:%.2f, cdh1:%.2f\n',theCell.gen,theCell.tdie,theCell.tdiv,theCell.v.OUTPUT2(end,84),theCell.v.OUTPUT2(end-1,126));
                    %cell divided
                    cell_count=cell_count+1;
                    %store stats is cell grew before dividing (it should)
                    if theCell.grew
                        sol.tp(theCell.gen) = sol.tp(theCell.gen)+1;
                        if (theCell.tdiv < 36*60 &&theCell.tdiv > 12*60) || (theCell.gen == 1 && (theCell.tdiv < 48*60 &&theCell.tdiv > 24*60))
                            sol.tp24(theCell.gen) = sol.tp24(theCell.gen) +1;
                            sol.h24counts(theCell.gen) = sol.h24counts(theCell.gen) +1;
                        end
                        sol.TdecideTimes{theCell.gen}{end+1}=theCell.tdecide;
                        sol.TdecideMothers{theCell.gen}{end+1}=theCell.parent;
                    else
                        sol.fn(theCell.gen) = sol.fn(theCell.gen)+1;
                        if theCell.tdiv < 24*60 || (theCell.gen == 1 && theCell.tdiv < 48*60)
                            sol.fn24(theCell.gen) = sol.fn24(theCell.gen) +1;
                            sol.h24counts(theCell.gen) = sol.h24counts(theCell.gen) +1;
                        end
                    end
                    
                    sol.fs(theCell.gen)=sol.fs(theCell.gen)+1;
                    %fprintf(1,'A cell divided at time %f it was %f minutes old. Total cells %d\n',theCell.tdiv+theCell.birthday,theCell.tdiv,length(cells)+length(cells2));
                    numdivs = numdivs+1;
                    %if theCell.tdiv<=0
                        %fprintf('cell tDiv <=0, tdiv: %d, tdie %d, tdecide %d\n',theCell.tdiv,theCell.tdie,theCell.tdecide);
                    %end
                    
                    sol.Tdivs(theCell.gen,ceil(theCell.tdiv/60))=sol.Tdivs(theCell.gen,ceil(theCell.tdiv/60))+1;
                    sol.divs(end+1)=ceil(theCell.tdiv/60);
                    for s=1:144
                        if size(theCell.v.OUTPUT2,1) > 100
                            for i = 1:1:100
                                sol.raverages(i,s,theCell.gen)=sol.raverages(i,s,theCell.gen)+theCell.v.OUTPUT2(ceil((size(theCell.v.OUTPUT2,1)*i)/100),s);
                            end
                        else
                            for i = 1:1:100
                                j=ceil((i*size(theCell.v.OUTPUT2,1))/100.0);
                                sol.raverages(i,s,theCell.gen)=sol.raverages(i,s,theCell.gen)+theCell.v.OUTPUT2(j,s);
                            end
                        end
                    end
                    
                    sol.TdivTimes{theCell.gen}{end+1}=theCell.tdiv;
                    sol.TdivMothers{theCell.gen}{end+1}=theCell.parent;
                    
                    %keep track of number of dividers
                    dividers=dividers+1;
                   
                    %r presumed 1 (no sampling) unless partition_CV is set
                    %r is ~= 1 mean asymmetric division
                    r = 1;
                    if experiment.PARTITION_CV>0
                        r = max(0,normrnd(1,experiment.PARTITION_CV));
                    end

                    dcell.v = getDaughterCellAsym(theCell.v,vo,r);
                    dcell.x = dcell.v.SPECIES{1};
                    
                    dcell.x(62) = experiment.L;
                    %however we need to update the cellular machine and size
                    dcell.x(134)=theCell.x(end,134)/2;
                    dcell.x(132)=theCell.x(end,132)/2;
                    
                    
                    dcell.x(126)=1.0; % set to 1 to avoid weirdness.       
                    %Also need empty out the contents of the nucleus
                    %(IkBa,b,d,e,RelA,P50,cRel)
                    for i = 3:3:21
                        dcell.x(i-1)= (dcell.x(i-1)+dcell.x(i))/2;
                        dcell.x(i)=(dcell.x(i-1)+dcell.x(i))/2;
                    end
                    %Empty out the nucleus continued (all dimer
                    %combinations)
                    for i = 23:2:61
                        dcell.x(i-1)= (dcell.x(i-1)+dcell.x(i))/2;
                        dcell.x(i)=(dcell.x(i-1)+dcell.x(i))/2;
                    end
                    %update params
                    dcell.birthday=theCell.tdiv+theCell.birthday;  % start at division time
                    dcell.gen = theCell.gen+1;
                    dcell.id = cell_id;
                    cell_id=cell_id+1;
                    dcell.parent = theCell.id;
                    dcell.v.SPECIES{1} = dcell.x;
                    dcell.founder= theCell.founder;
                    if numel(cells2) < N*100
                        cells2{end+1} = dcell;  % append
                    end
                    %Daughter 2
                    %Asymmetric division partition cells such that if one
                    %daughter is large the other is small
                    r = (1-(r-1));
                    dcell2.v = getDaughterCellAsym(theCell.v,vo,r);
                    dcell2.x = dcell2.v.SPECIES{1};
                    
                    dcell2.x(62) = experiment.L;

                    %however we need to update the cellular machine and size
                    dcell2.x(134)=theCell.x(end,134)/2;
                    dcell2.x(132)=theCell.x(end,132)/2;
                    dcell2.x(126) = 1.0; % set to 1 to avoid weirdness
                    %Also need empty out the contents of the nucleus
                    %(IkBa,b,d,e,RelA,P50,cRel)
                    for i = 3:3:21
                        dcell2.x(i-1)= (dcell2.x(i-1)+dcell2.x(i))/2;
                        dcell2.x(i)=(dcell2.x(i-1)+dcell2.x(i))/2;
                    end
                    %Empty out the nucleus continued (all dimer
%                     %combinations)
                    for i = 23:2:61
                        dcell2.x(i-1)= (dcell2.x(i-1)+dcell2.x(i))/2;
                        dcell2.x(i)=(dcell2.x(i-1)+dcell2.x(i))/2;
                    end

                    %%% L is constant
                    dcell2.x(62) = experiment.L;
                     %update params
                    dcell2.birthday=theCell.tdiv+theCell.birthday;  % start at division time
                    dcell2.gen =theCell.gen+1;
                    dcell2.id = cell_id;
                    cell_id=cell_id+1;
                    dcell2.parent = theCell.id;
                    dcell2.founder= theCell.founder;
                    dcell2.v.SPECIES{1} = dcell2.x;
                    %dcell2.v.OUTPUT2 = [];
                    if numel(cells2) < N*100
                        cells2{end+1} = dcell2;  % append
                    end
                    
                    %some species numbers to keep track of which species is which later in
                    %the program.
                    RelA50nIndex=33;
                    cRel50nIndex=53;
                    RelA50TotalIndex=[32:41];
                    cRel50TotalIndex=[52:61];
                    RelAnIndex=[15,23:2:41];
                    cRelnIndex=[21,53:2:61];
                    RelATotalIndex=[14,15,22:41];
                    cRelTotalIndex=[20,21,52:61];
                    BclXLIndex=[61+33];
                    CYCDIndex=[61+58+3];
                    MycIndex=[61+58+24];
                    IkBaIndex=[1];
                    IkBbIndex=[4];
                    IkBeIndex=[7];
                    IkBdIndex=[10];
                    MassIndex=[61+28+15];
                    cParpIndex=[84];
                    ParpIndex=[82];
                    c3ParpIndex=[83];
                    cdh1Index=[126];
                    
                    if totalCellNumberCurrentlyProcessed+length(cellsToProcess)>size(sol.RelA50n,2)
                        %we need to add more space for new cell timecourses to
                        %be stored.
                        size(sol.RelA50n);
                        ceil(maxTime/30);
                        length(cellsToProcess);
                        sol.RelA50n=[sol.RelA50n,zeros(size(sol.RelA50n,1),length(cellsToProcess))];
                        sol.cRel50n=[sol.cRel50n,zeros(size(sol.cRel50n,1),length(cellsToProcess))];
                        sol.RelA50Total=[sol.RelA50Total,zeros(size(sol.RelA50Total,1),length(cellsToProcess))];
                        sol.cRel50Total=[sol.cRel50Total,zeros(size(sol.cRel50Total,1),length(cellsToProcess))];
                        sol.RelAn=[sol.RelAn,zeros(size(sol.RelAn,1),length(cellsToProcess))];
                        sol.cReln=[sol.RelAn,zeros(size(sol.cReln,1),length(cellsToProcess))];
                        sol.RelATotal=[sol.RelATotal,zeros(size(sol.RelATotal,1),length(cellsToProcess))];
                        sol.cRelTotal=[sol.cRelTotal,zeros(size(sol.cRelTotal,1),length(cellsToProcess))];
                        sol.BclXL=[sol.BclXL,zeros(size(sol.BclXL,1),length(cellsToProcess))];
                        sol.CYCD=[sol.CYCD,zeros(size(sol.CYCD,1),length(cellsToProcess))];
                        sol.Myc=[sol.Myc,zeros(size(sol.Myc,1),length(cellsToProcess))];
                        sol.IkBa=[sol.IkBa,zeros(size(sol.IkBa,1),length(cellsToProcess))];
                        sol.IkBb=[sol.IkBb,zeros(size(sol.IkBb,1),length(cellsToProcess))];
                        sol.IkBe=[sol.IkBe,zeros(size(sol.IkBe,1),length(cellsToProcess))];
                        sol.IkBd=[sol.IkBd,zeros(size(sol.IkBd,1),length(cellsToProcess))];
                        sol.Mass=[sol.Mass,zeros(size(sol.Mass,1),length(cellsToProcess))];
                        sol.lineageInfo=[sol.lineageInfo,zeros(size(sol.lineageInfo,1),length(cellsToProcess))];
                        sol.cParp=[sol.cParp,zeros(size(sol.cParp,1),length(cellsToProcess))];
                        sol.Parp=[sol.Parp,zeros(size(sol.Parp,1),length(cellsToProcess))];
                        sol.c3Parp=[sol.c3Parp,zeros(size(sol.c3Parp,1),length(cellsToProcess))];
                        sol.cdh1=[sol.cdh1,zeros(size(sol.cdh1,1),length(cellsToProcess))];

                    end


                    birthday=ceil(theCell.birthday/30);
                    deathTime=ceil(theCell.tdiv/30)+birthday;
                    birthday=birthday+1;
                    
                    sol.RelA50n(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,RelA50nIndex),2);
                    sol.cRel50n(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,cRel50nIndex),2);
                    sol.RelA50Total(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,RelA50TotalIndex),2);
                    sol.cRel50Total(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,cRel50TotalIndex),2);
                    sol.RelAn(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,RelAnIndex),2);
                    sol.cReln(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,cRelnIndex),2);
                    sol.RelATotal(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,RelATotalIndex),2);
                    sol.cRelTotal(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,cRelTotalIndex),2);
                    sol.BclXL(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,BclXLIndex),2);
                    sol.CYCD(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,CYCDIndex),2);
                    sol.Myc(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,MycIndex),2);
                    sol.IkBa(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,IkBaIndex),2);
                    sol.IkBb(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,IkBbIndex),2);
                    sol.IkBe(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,IkBeIndex),2);
                    sol.IkBd(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,IkBdIndex),2);
                    sol.Mass(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,MassIndex),2);
                    sol.cParp(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,cParpIndex),2);
                    sol.Parp(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,ParpIndex),2);
                    sol.c3Parp(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,c3ParpIndex),2);
                    sol.cdh1(birthday:deathTime,totalCellNumberCurrentlyProcessed)=sum(theCell.v.OUTPUT2(1:30:end,cdh1Index),2);
                    
                    
                    sol.cParp(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.Parp(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.c3Parp(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.cdh1(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    
                    sol.RelA50n(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.cRel50n(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.RelA50Total(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.cRel50Total(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.RelAn(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.cReln(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.RelATotal(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.cRelTotal(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.BclXL(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.CYCD(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.Myc(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.IkBa(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.IkBb(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.IkBe(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.IkBd(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    sol.Mass(end,totalCellNumberCurrentlyProcessed)=theCell.gen;
                    
                    sol.lineageInfo(1,totalCellNumberCurrentlyProcessed)= theCell.id;
                    sol.lineageInfo(2,totalCellNumberCurrentlyProcessed)=theCell.parent;
                    sol.lineageInfo(3,totalCellNumberCurrentlyProcessed)=theCell.birthday+theCell.tdiv;
                    
                    totalCellNumberCurrentlyProcessed=totalCellNumberCurrentlyProcessed+1;
                    cells(1)=[];
                end
            end
            sol.divbag{end+1}=sol.divs;
            sol.diebag{end+1}=sol.deaths;
            sol.decidebag{end+1}=sol.decides;
            for s=1:144
                for i=1:100
                    sol.qaverages(i,s,Cgen)=sol.qaverages(i,s,Cgen)/nondividers;
                    sol.raverages(i,s,Cgen)=sol.raverages(i,s,Cgen)/dividers;
                end
            end
            %cells = cells2;
            %let's save the cells in arrays of no more than 250
            if length(cells2)<1
                %fprintf(1,'\nNo Cells To Save\n');
            elseif length(cells2)<150
                experiment_label_Cells=sprintf('Cells-%s.mat',datestr(now,'mmmm dd, yyyy HH-MM-SS-FFF'));
                fileToSave=fullfile('unprocessedCells',experiment_label_Cells);
                cells=cells2;
                save('-v7.3',fileToSave,'cells');
                %fprintf(1,'\nSaving %d New Unprocessed Cells to %s\n', length(cells2),fileToSave); 
            else
                numberOfFiles=0;
                for cellIndex=1:150:length(cells2)
                    experiment_label_Cells=sprintf('Cells-%s.mat',datestr(now,'mmmm dd, yyyy HH-MM-SS-FFF'));
                    fileToSave=fullfile('unprocessedCells',experiment_label_Cells);
                    indexToSaveFrom=cellIndex;
                    indexToSaveTo=cellIndex+149;
                    if length(cells2)<indexToSaveTo
                        indexToSaveTo=length(cells2);
                    end
                    variableToSave=cells2(indexToSaveFrom:indexToSaveTo);
                    cells=variableToSave;
                    save('-v7.3',fileToSave,'cells');
                    numberOfFiles=numberOfFiles+1;
                end
                fprintf(1,'\nSaved %d New Unprocessed Cells to %d files\n', length(cells2),numberOfFiles); 
            end
            if totalCellNumberCurrentlyProcessed+length(cellsToProcess)>size(sol.RelA50n,2)
                %we need to add more space for new cell timecourses
                %to be stored.

            sol.RelA50n=[sol.RelA50n,zeros(size(sol.RelA50n,1),length(cellsToProcess))];
            sol.cRel50n=[sol.cRel50n,zeros(size(sol.cRel50n,1),length(cellsToProcess))];
            sol.RelA50Total=[sol.RelA50Total,zeros(size(sol.RelA50Total,1),length(cellsToProcess))];
            sol.cRel50Total=[sol.cRel50Total,zeros(size(sol.cRel50Total,1),length(cellsToProcess))];
            sol.RelAn=[sol.RelAn,zeros(size(sol.RelAn,1),length(cellsToProcess))];
            sol.cReln=[sol.RelAn,zeros(size(sol.cReln,1),length(cellsToProcess))];
            sol.RelATotal=[sol.RelATotal,zeros(size(sol.RelATotal,1),length(cellsToProcess))];
            sol.cRelTotal=[sol.cRelTotal,zeros(size(sol.cRelTotal,1),length(cellsToProcess))];
            sol.BclXL=[sol.BclXL,zeros(size(sol.BclXL,1),length(cellsToProcess))];
            sol.CYCD=[sol.CYCD,zeros(size(sol.CYCD,1),length(cellsToProcess))];
            sol.Myc=[sol.Myc,zeros(size(sol.Myc,1),length(cellsToProcess))];
            sol.IkBa=[sol.IkBa,zeros(size(sol.IkBa,1),length(cellsToProcess))];
            sol.IkBb=[sol.IkBb,zeros(size(sol.IkBb,1),length(cellsToProcess))];
            sol.IkBe=[sol.IkBe,zeros(size(sol.IkBe,1),length(cellsToProcess))];
            sol.IkBd=[sol.IkBd,zeros(size(sol.IkBd,1),length(cellsToProcess))];
            sol.Mass=[sol.Mass,zeros(size(sol.Mass,1),length(cellsToProcess))];
            sol.lineageInfo=[sol.lineageInfo,zeros(size(sol.lineageInfo,1),length(cellsToProcess))];
            sol.cParp=[sol.cParp,zeros(size(sol.Mass,1),length(cellsToProcess))];
            sol.Parp=[sol.Parp,zeros(size(sol.Mass,1),length(cellsToProcess))];
            sol.c3Parp=[sol.c3Parp,zeros(size(sol.Mass,1),length(cellsToProcess))];
            sol.cdh1=[sol.cdh1,zeros(size(sol.Mass,1),length(cellsToProcess))];

            end
            
            fileToSave=fullfile('Checkpoints', 'solutionOut.mat');
            save('-v7.3',fileToSave,'sol');
            fprintf(1,'\n Moving file to processed folder: %s\n',fullfile('processedCells',cellsToProcess(k).name));
            movefile(fullfile('unprocessedCells',cellsToProcess(k).name),fullfile('processedCells',cellsToProcess(k).name));         
            
        end
    end
        
    fprintf(1,'Processing stats\n');
    
    sol.RelA50n( :, ~any(sol.RelA50n,1) ) = [];
    sol.cRel50n( :, ~any(sol.cRel50n,1) ) = [];
    sol.RelA50Total( :, ~any(sol.RelA50Total,1) ) = [];
    sol.cRel50Total( :, ~any(sol.cRel50Total,1) ) = [];
    sol.RelAn( :, ~any(sol.RelAn,1) ) = [];
    sol.cReln( :, ~any(sol.cReln,1) ) = [];
    sol.RelATotal( :, ~any(sol.RelATotal,1) ) = [];
    sol.cRelTotal( :, ~any(sol.cRelTotal,1) ) = [];
    sol.BclXL( :, ~any(sol.BclXL,1) ) = [];
    sol.CYCD( :, ~any(sol.CYCD,1) ) = [];
    sol.Myc( :, ~any(sol.Myc,1) ) = [];
    sol.IkBa( :, ~any(sol.IkBa,1) ) = [];
    sol.IkBb( :, ~any(sol.IkBb,1) ) = [];
    sol.IkBe( :, ~any(sol.IkBe,1) ) = [];
    sol.IkBd( :, ~any(sol.IkBd,1) ) = [];
    sol.Mass( :, ~any(sol.Mass,1) ) = [];
    sol.cParp(:, ~any(sol.cParp,1) ) = [];
    sol.Parp(:, ~any(sol.Parp,1) ) = [];
    sol.c3Parp(:, ~any(sol.c3Parp,1) ) = [];
    sol.cdh1(:, ~any(sol.cdh1,1) ) = [];

    sol.lineageInfo( :, ~any(sol.lineageInfo,1) ) = [];
    
    %Post processing of stats
    for i=1:maxGens
        sol.counts(:,i)=sol.counts(:,i)-1;
    end
    accuracies=zeros(maxGens,1);
    logErrors=zeros(maxGens,2);
    statistics=zeros(maxGens,20);
    for g=1:maxGens
        if sol.integralCounts(g) > 0
            %lets compute the stats
            sol.fs(g)=sol.fs(g)/sol.integralCounts(g);
            sol.gs(g)=sol.gs(g)/sol.integralCounts(g);
            sol.tp(g)=sol.tp(g)/sol.integralCounts(g);
            sol.tn(g)=sol.tn(g)/sol.integralCounts(g);
            sol.fp(g)=sol.fp(g)/sol.integralCounts(g);
            sol.fn(g)=sol.fn(g)/sol.integralCounts(g);
            sol.tp24(g)=sol.tp24(g)/sol.h24counts(g);
            sol.tn24(g)=sol.tn24(g)/sol.h24counts(g);
            sol.fp24(g)=sol.fp24(g)/sol.h24counts(g);
            sol.fn24(g)=sol.fn24(g)/sol.h24counts(g);
            statistics(g,1)=sol.integralCounts(g);
            statistics(g,2)=sol.gs(g);
            statistics(g,3)=sol.fs(g);
            statistics(g,4)=sol.tp(g);
            statistics(g,5)=sol.tn(g);
            statistics(g,6)=sol.fp(g);
            statistics(g,7)=sol.fn(g);
            accuracies(g)=(sol.tp(g)+sol.tn(g));
            statistics(g,8)=accuracies(g);
            if ~ isempty(sol.divbag{g})
                sol.maxDivGen=g;
                %lets fit a lognormal to the divs
                vector = sol.divbag{g};
                [parmhat,parmci] = lognfit(vector(:));
                [M,V]=lognstat(parmhat(1),parmhat(2));
                TdivFits(g,1)=M;
                TdivFits(g,2)=sqrt(V);
                [Mlo,Vlo]=lognstat(parmci(1,1),parmci(2,2));
                [Mhi,Vhi]=lognstat(parmci(1,2),parmci(2,1));
                TdivFits(g,3)=Mlo;
                TdivFits(g,4)=Mhi;
                TdivFits(g,5)=sqrt(Vlo);
                TdivFits(g,6)=sqrt(Vhi);
                logErrors(g,1)=(((Mhi-Mlo)/M)+((Vhi-Vlo)/V))/2;
                for i = 1:6
                    statistics(g,8+i)=TdivFits(g,i);
                end
            end
            if ~ isempty(sol.diebag{g}) 
                sol.maxDieGen=g;
                %lets fit a lognormal to the divs
                vector = sol.diebag{g};
                [parmhat,parmci] = lognfit(vector(:));
                [M,V]=lognstat(parmhat(1),parmhat(2));
                TdieFits(g,1)=M;
                TdieFits(g,2)=sqrt(V);
                [Mlo,Vlo]=lognstat(parmci(1,1),parmci(2,2));
                [Mhi,Vhi]=lognstat(parmci(1,2),parmci(2,1));
                TdieFits(g,3)=Mlo;
                TdieFits(g,4)=Mhi;
                TdieFits(g,5)=sqrt(Vlo);
                TdieFits(g,6)=sqrt(Vhi);
                logErrors(g,2)=(((Mhi-Mlo)/M)+((Vhi-Vlo)/V))/2;
                for i = 1:6
                    statistics(g,14+i)=TdieFits(g,i);
                end
            end
        end    
    end
    sol.statistics = statistics;
    sol.logErrors = logErrors;
    sol.TdivFits = TdivFits;
    sol.TdieFits = TdieFits;   
end

function saveSolutions(solutions,N,maxTime,maxGens,label,paramString)
    experiment_label=sprintf('%s %s',label,datestr(now,'mmmm dd, yyyy HH-MM-SS-FFF'));
    mkdir(experiment_label);
    copyfile('*.m',experiment_label);
    ts = 1:30:maxTime;
    ts=ts./60;
    totals = zeros(length(solutions),length(ts));
    for r = 1:length(solutions)
        for t = 1:length(ts)
            totals(r,t) = sum(solutions{r}.counts(t,:));
        end
    end
    handle = fopen(sprintf('%s\\Parameters.txt',experiment_label),'w+');
    fprintf(handle,paramString);
    outarray = zeros((length(solutions)+1)*(maxGens+1)+1,length(ts));
    outarray(1,:) = ts';
    for g = 1: maxGens
       means = zeros(length(ts),1);
       for r = 1: length(solutions)
            means=means+solutions{r}.counts(:,g);
       end
       outarray(g+1,:)=means(:)./length(solutions);
    end
    outarray(maxGens+2,:)=sum(outarray(2:g+1,:));
    for r = 1: length(solutions)
        for g = 1: maxGens
            outarray((r)*(maxGens+1)+g+1,:)=solutions{r}.counts(:,g);
        end
        outarray((r)*(maxGens+1)+maxGens+2,:)=totals(r,:);
    end
    csvwrite(sprintf('%s\\counts.csv',experiment_label),outarray');

    outarray = zeros(length(solutions)*maxGens*2,100);
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g,:)=solutions{r}.qaverages(:,134,g);
        end
    end
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g+maxGens,:)=solutions{r}.raverages(:,134,g);
        end
    end
    csvwrite(sprintf('%s\\Masses.csv',experiment_label),outarray');
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g,:)=solutions{r}.qaverages(:,33,g);
        end
    end
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g+maxGens,:)=solutions{r}.raverages(:,33,g);
        end
    end
    csvwrite(sprintf('%s\\RelA50n.csv',experiment_label),outarray');   
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g,:)=solutions{r}.qaverages(:,53,g);
        end
    end
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g+maxGens,:)=solutions{r}.raverages(:,53,g);
        end
    end
    csvwrite(sprintf('%s\\cRel50n.csv',experiment_label),outarray'); 
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g,:)=solutions{r}.qaverages(:,94,g);
        end
    end
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g+maxGens,:)=solutions{r}.raverages(:,94,g);
        end
    end
    csvwrite(sprintf('%s\\BclXL.csv',experiment_label),outarray'); 
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g,:)=solutions{r}.qaverages(:,143,g);
        end
    end
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g+maxGens,:)=solutions{r}.raverages(:,143,g);
        end
    end
    csvwrite(sprintf('%s\\Myc.csv',experiment_label),outarray'); 
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g,:)=solutions{r}.qaverages(:,122,g);
        end
    end
    for r= 1: length(solutions)
        for g=1:maxGens
            outarray((r-1)*maxGens*2+g+maxGens,:)=solutions{r}.raverages(:,122,g);
        end
    end
    csvwrite(sprintf('%s\\CYCD.csv',experiment_label),outarray'); 
    
    
    
    %writing raw cell timecourses of species to individual files here, one
    %file for each species. One file for each replicate.
    mkdir(fullfile(experiment_label,'MetaboliteTimeCourses'));
    
    for r= 1: length(solutions)   
        
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('RelA50n-rep',num2str(r),'.CSV')),solutions{r}.RelA50n);
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('cRel50n-rep',num2str(r),'.CSV')),solutions{r}.cRel50n);
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('RelA50Total-rep',num2str(r),'.CSV')),solutions{r}.RelA50Total);
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('cRel50Total-rep',num2str(r),'.CSV')),solutions{r}.cRel50Total);
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('RelAn-rep',num2str(r),'.CSV')),solutions{r}.RelAn);
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('cReln-rep',num2str(r),'.CSV')),solutions{r}.cReln);
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('RelATotal-rep',num2str(r),'.CSV')),solutions{r}.RelATotal);
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('cRelTotal-rep',num2str(r),'.CSV')),solutions{r}.cRelTotal);
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('BclXL-rep',num2str(r),'.CSV')),solutions{r}.BclXL);
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('CYCD-rep',num2str(r),'.CSV')),solutions{r}.CYCD);
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('Myc-rep',num2str(r),'.CSV')),solutions{r}.Myc);
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('IkBa-rep',num2str(r),'.CSV')),solutions{r}.IkBa);
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('IkBb-rep',num2str(r),'.CSV')),solutions{r}.IkBb);
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('IkBe-rep',num2str(r),'.CSV')),solutions{r}.IkBe);
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('IkBd-rep',num2str(r),'.CSV')),solutions{r}.IkBd);
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('Mass-rep',num2str(r),'.CSV')),solutions{r}.Mass);
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('cParp-rep',num2str(r),'.CSV')),solutions{r}.cParp);        
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('Parp-rep',num2str(r),'.CSV')),solutions{r}.Parp);        
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('c3Parp-rep',num2str(r),'.CSV')),solutions{r}.c3Parp);        
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('cdh1-rep',num2str(r),'.CSV')),solutions{r}.cdh1);        
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('lineageInfo-rep',num2str(r),'.CSV')),solutions{r}.lineageInfo);
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('generationOverTime-rep',num2str(r),'.CSV')),solutions{r}.generationsOverTime);
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('rangeDistribution-rep',num2str(r),'.CSV')),solutions{r}.rangeDistribution);

        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('maxGenDistribution-rep',num2str(r),'.CSV')),solutions{r}.maxGenDistribution);
        csvwrite(fullfile(experiment_label,'MetaboliteTimeCourses',strcat('minGenDistribution-rep',num2str(r),'.CSV')),solutions{r}.minGenDistribution);

        
    end
    
    
    outarray = zeros(maxGens,(length(solutions)+1)*9);
    for r = 1: length(solutions)
        for g = 1: maxGens
            outarray(g,(r)*9+1)=solutions{r}.fs(g);
            outarray(g,(r)*9+2)=solutions{r}.tp(g);
            outarray(g,(r)*9+3)=solutions{r}.fp(g);
            outarray(g,(r)*9+4)=solutions{r}.tn(g);
            outarray(g,(r)*9+5)=solutions{r}.fn(g);
            outarray(g,(r)*9+6)=solutions{r}.tp24(g);
            outarray(g,(r)*9+7)=solutions{r}.fp24(g);
            outarray(g,(r)*9+8)=solutions{r}.tn24(g);
            outarray(g,(r)*9+9)=solutions{r}.fn24(g);
            outarray(g, 1)=outarray(g, 1)+solutions{r}.fs(g)/length(solutions);
            outarray(g, 2)=outarray(g, 2)+solutions{r}.tp24(g)/length(solutions);
            outarray(g, 3)=outarray(g, 3)+solutions{r}.fp24(g)/length(solutions);
            outarray(g, 4)=outarray(g, 4)+solutions{r}.tn24(g)/length(solutions);
            outarray(g, 5)=outarray(g, 5)+solutions{r}.fn24(g)/length(solutions);
            outarray(g, 6)=outarray(g, 6)+(solutions{r}.tp24(g)/(solutions{r}.tp24(g)+solutions{r}.fp24(g)))/length(solutions);
            outarray(g, 7)=outarray(g, 7)+(solutions{r}.fp24(g)/(solutions{r}.tp24(g)+solutions{r}.fp24(g)))/length(solutions);
            outarray(g, 8)=outarray(g, 8)+(solutions{r}.tn24(g)/(solutions{r}.tn24(g)+solutions{r}.fn24(g)))/length(solutions);
            outarray(g, 9)=outarray(g, 9)+(solutions{r}.fn24(g)/(solutions{r}.tn24(g)+solutions{r}.fn24(g)))/length(solutions);

        end
    end
    csvwrite(sprintf('%s\\Fs.csv',experiment_label),outarray);    
    handle = fopen(sprintf('%s\\FateTimes.csv',experiment_label),'w+');
    for r = 1:length(solutions)
        for g = 1:maxGens
            fprintf(handle,',,%dTdec%d,,%dTdiv%d,,%dTdie%d',r,g,r,g,r,g);
        end
    end
    fprintf(handle,',AllTdec0,AllTdiv0,AllTdie0,AllTdec1+,AllTdiv1+,AllTdie1+\n');
    AllTdec0 = cell(1);
    AllTdiv0 = cell(1);
    AllTdie0 = cell(1);
    AllTdec1 = cell(1);
    AllTdiv1 = cell(1);
    AllTdie1 = cell(1);
    for r = 1: length(solutions)
        for t = 1:numel(solutions{r}.TdecideTimes{1})
            if numel(solutions{r}.TdecideTimes{1}{1}) > 0
                AllTdec0{end+1} = solutions{r}.TdecideTimes{1}{t};
            end
        end
        for t = 1:numel(solutions{r}.TdivTimes{1})
            if numel(solutions{r}.TdivTimes{1}{1}) > 0
                AllTdiv0{end+1} = solutions{r}.TdivTimes{1}{t};
            end
        end
        for t = 1:numel(solutions{r}.TdieTimes{1})
            if numel(solutions{r}.TdieTimes{1}{1}) > 0
                AllTdie0{end+1} = solutions{r}.TdieTimes{1}{t};
            end
        end

        for g = 2:maxGens
            for t = 1:numel(solutions{r}.TdecideTimes{g})
                if numel(solutions{r}.TdecideTimes{g}{t}) > 0
                    AllTdec1{end+1} = solutions{r}.TdecideTimes{g}{t};
                end
            end
            for t = 1:numel(solutions{r}.TdivTimes{g})
                if numel(solutions{r}.TdivTimes{g}{t}) > 0
                    AllTdiv1{end+1} = solutions{r}.TdivTimes{g}{t};
                end
            end
            for t = 1:numel(solutions{r}.TdieTimes{g})
                if numel(solutions{r}.TdieTimes{g}{t}) > 0
                    AllTdie1{end+1} = solutions{r}.TdieTimes{g}{t};
                end
            end
        end
    end
    for row = 1:100000
       rowcount=0;
       for r = 1: length(solutions)
            for i = 1:maxGens
                if row <= numel(solutions{r}.TdecideTimes{i}) 
                   fprintf(handle,',%d,%2.2f',solutions{r}.TdecideMothers{i}{row},solutions{r}.TdecideTimes{i}{row}/60.0);
                   rowcount=rowcount+1;
                else
                    fprintf(handle,',,');
                end
                if row <= numel(solutions{r}.TdivTimes{i})
                   fprintf(handle,',%d,%2.2f',solutions{r}.TdivMothers{i}{row},solutions{r}.TdivTimes{i}{row}/60.0);
                   rowcount=rowcount+1;
                else
                    fprintf(handle,',,');
                end
                if row <= numel(solutions{r}.TdieTimes{i})
                   fprintf(handle,',%d,%2.2f',solutions{r}.TdieMothers{i}{row},solutions{r}.TdieTimes{i}{row}/60.0);
                   rowcount=rowcount+1;
                else
                    fprintf(handle,',,');
                end
            end
       end
       if row <= numel(AllTdec0)
           fprintf(handle,',%2.2f',AllTdec0{row}/60.0);
           rowcount=rowcount+1;
       else
           fprintf(handle,',');
       end
       if row <= numel(AllTdiv0)
           fprintf(handle,',%2.2f',AllTdiv0{row}/60.0);
           rowcount=rowcount+1;
       else
           fprintf(handle,',');
       end
       if row <= numel(AllTdie0)
           fprintf(handle,',%2.2f',AllTdie0{row}/60.0);
           rowcount=rowcount+1;
       else
           fprintf(handle,',');
       end
       if row <= numel(AllTdec1)
           fprintf(handle,',%2.2f',AllTdec1{row}/60.0);
           rowcount=rowcount+1;
       else
           fprintf(handle,',');
       end
       if row <= numel(AllTdiv1)
           fprintf(handle,',%2.2f',AllTdiv1{row}/60.0);
           rowcount=rowcount+1;
       else
           fprintf(handle,',');
       end
       if row <= numel(AllTdie1)
           fprintf(handle,',%2.2f',AllTdie1{row}/60.0);
           rowcount=rowcount+1;
       else
           fprintf(handle,',');
       end

       fprintf(handle,'\n');
       if rowcount == 0
           break;
       end
    end
    fclose(handle);
    %csvwrite(sprintf('%s\\FateTimes.csv',experiment_label),[cell2mat(sol.TdecideTimes),cell2mat(sol.TdivTimes),cell2mat(sol.TdieTimes)]);
    %end
    %save Tdecides, Tdivs, Tides 
    %fprintf(1,'Saved all masses and counts to %s\n',experiment_label);
end


% Make a copy of a handle object.
function new = copy(this)
    % Instantiate new object of the same class.
    new = feval(class(this));
    % Copy all non-hidden properties.
    p = fieldnames(struct(this));
    for i = 1:length(p)
        new.(p{i}) = this.(p{i});
    end
end

function c2 = duplicate(sol)
    c2 = struct;   
    c2.NFKB_CYCD_T=sol.NFKB_CYCD_T;
    c2.NFKB_BCL2_T=sol.NFKB_BCL2_T;
    c2.NFKB_MYC_T=sol.NFKB_MYC_T;
    c2.GROWTH_T=sol.GROWTH_T;
    c2.RATE_CV=sol.RATE_CV;
    c2.CONC_CV=sol.CONC_CV;
    c2.PARTITION_CV=sol.PARTITION_CV;
    c2.VOL_CV=sol.VOL_CV;
    c2.IKK_CV=sol.IKK_CV;
    c2.GMSyn=sol.GMSyn;
    c2.GMDeg=sol.GMDeg;
    c2.MassGrowth=sol.MassGrowth;
    c2.MassDecay=sol.MassDecay;
    c2.CYCDtSyn=    sol.CYCDtSyn;
    c2.CYCDtDeg=    sol.CYCDtDeg;
    c2.BCL2tSyn=    sol.BCL2tSyn;
    c2.BCL2tDeg=	sol.BCL2tDeg;
    c2.MYCSyn=      sol.MYCSyn;
    c2.MYCDeg=      sol.MYCDeg;
    c2.MYCtSyn=     sol.MYCtSyn;
    c2.MYCtDeg=     sol.MYCtDeg;
    c2.L=           sol.L;
    c2.STIMULUS=    sol.STIMULUS;
    c2.CREL=        sol.CREL;
    c2.cRelFeedback=sol.cRelFeedback;
    c2.EPS=         sol.EPS;
    c2.TOTAL_IKK=   sol.TOTAL_IKK;
    c2.pre_RATE_CV = sol.pre_RATE_CV;
    c2.pre_CONC_CV = sol.pre_CONC_CV;
    c2.maxTime=sol.maxTime;  
end



function v = getRandomCell(v)
        %--- Randomization of NFkB syn/deg params
        for i = [1,2,3,4,5,11,12,13,14]          
            for j = 1:size(v.pi(i,:),2)
                v.pi(i,j)=max(0,normrnd(v.pi(i,j),v.pi(i,j)*v.pre_RATE_CV));
            end
        end
        for i = [1,2,3,4,5]         
            for j = 1:size(v.pm(i,:),2)
                v.pm(i,j)=max(0,normrnd(v.pm(i,j),v.pm(i,j)*v.pre_RATE_CV));
            end
        end
        for i = 9:12          
            for j = 1:size(v.pd(i,:),2)
                v.pd(i,j)=max(0,normrnd(v.pd(i,j),v.pd(i,j)*v.pre_RATE_CV));
            end
        end
        %--- apoptosis syn/deg
        for i = 71:115
            v.k(i)= max(0,normrnd(v.k(i),v.k(i)*v.pre_RATE_CV));
        end
        %--- cell cycle syn/deg
        for i = [8,9,12:15,18:34,47:67,74:76,78:86]
            v.k2(i)= max(0,normrnd(v.k2(i),v.k2(i)*v.pre_RATE_CV));
        end
        %--- cell cycle species randomization
        theMask = zeros(size(v.SPECIES{1},1),1);
        theMask(137:142) = 1;
        v.SPECIES{1} = getRandomSpeciesMaskLognormal(v.SPECIES{1},theMask,v.pre_CONC_CV,0);
end


function v2 = getDaughterCell(v,vo,r)
        v2 = copy(v);
        for i = [1,2,3,4,5,11,12,13,14]          
            for j = 1:size(v.pi(i,:),2)
                v2.pi(i,j)=max(0,(normrnd(v.pi(i,j),v.pi(i,j)*v.RATE_CV)+normrnd(vo.pi(i,j),vo.pi(i,j)*vo.RATE_CV))/2);
            end
        end
        for i = [1,2,3,4,5]         
            for j = 1:size(v.pm(i,:),2)
                v2.pm(i,j)=max(0,(normrnd(v.pm(i,j),v.pm(i,j)*v.RATE_CV)+normrnd(vo.pm(i,j),vo.pm(i,j)*vo.RATE_CV))/2);
            end
        end
        for i = 9:12          
            for j = 1:size(v.pd(i,:),2)
                v2.pd(i,j)=max(0,(normrnd(v.pd(i,j),v.pd(i,j)*v.RATE_CV)+normrnd(vo.pd(i,j),vo.pd(i,j)*vo.RATE_CV))/2);
            end
        end
        %--- apoptosis syn/deg
        for i = 71:115
            v2.k(i,j)=max(0,(normrnd(v.k(i),v.k(i)*v.RATE_CV)+normrnd(vo.k(i),vo.k(i)*vo.RATE_CV))/2);
        end
        %--- cell cycle syn/deg
        for i = [8,9,12:15,18:34,47:67,74:76,78:86]
            v2.k2(i,j)=max(0,(normrnd(v.k2(i),v.k2(i)*v.RATE_CV)+normrnd(vo.k2(i),vo.k2(i)*vo.RATE_CV))/2);
        end
        v2.SPECIES{1}(132)=v.SPECIES{1}(132)*r;
        v2.SPECIES{1}(134)=v.SPECIES{1}(134)*r;
        %==== IKK Input Curve
        if vo.IKK_CV > 0
            sigma= sqrt(log(1+(vo.IKK_CV^2)));
            mu= log(140)-0.5*sigma^2; 
            rnd = lognrnd(mu,sigma);
            v2.TOTAL_IKK=(v.TOTAL_IKK+rnd)/2;
        end
        %--- cell cycle species randomization
        %added by Simon
        theMask = zeros(size(v2.SPECIES{1},2),1);
        theMask(137:142) = 1;
        v2.SPECIES{1} = getRandomSpeciesMaskLognormal(v2.SPECIES{1},theMask,v.CONC_CV,0);
end

function v2 = getDaughterCellAsym(v,vo,r)
        v2 = copy(v);
        for i = [1,2,3,4,5,11,12,13,14]          
            for j = 1:size(v.pi(i,:),2)
                %v2.pi(i,j)=max(0,(normrnd(v.pi(i,j),v.pi(i,j)*v.RATE_CV)+normrnd(vo.pi(i,j),vo.pi(i,j)*vo.RATE_CV))/2);
                v2.pi(i,j)=max(0,(normrnd(v.pi(i,j),v.pi(i,j)*v.RATE_CV)));
            end
        end
        for i = [1,2,3,4,5]         
            for j = 1:size(v.pm(i,:),2)
                v2.pm(i,j)=max(0,(normrnd(v.pm(i,j),v.pm(i,j)*v.RATE_CV)));
            end
        end
        for i = 9:12          
            for j = 1:size(v.pd(i,:),2)
                v2.pd(i,j)=max(0,(normrnd(v.pd(i,j),v.pd(i,j)*v.RATE_CV)));
            end
        end
        %--- apoptosis syn/deg
        for i = 71:115
            v2.k(i,j)=max(0,(normrnd(v.k(i),v.k(i)*v.RATE_CV)));
        end
        %--- cell cycle syn/deg
        for i = [8,9,12:15,18:34,47:67,74:76,78:86]
            v2.k2(i,j)=max(0,(normrnd(v.k2(i),v.k2(i)*v.RATE_CV)));
        end
        %adjusted to randomize all species
        v2.SPECIES{1}(:)=v.SPECIES{1}(:).*r;
        %v2.SPECIES{1}(:)=v.SPECIES{1}(:).*r;
        %==== IKK Input Curve
        if vo.IKK_CV > 0
            sigma= sqrt(log(1+(vo.IKK_CV^2)));
            mu= log(140)-0.5*sigma^2; 
            rnd = lognrnd(mu,sigma);
            v2.TOTAL_IKK=(v.TOTAL_IKK+rnd)/2;
        end
        %--- cell cycle species randomization
        %added by Simon
        %theMask = zeros(size(v2.SPECIES{1},2),1);
        %theMask(137:142) = 1;
        %v2.SPECIES{1} = getRandomSpeciesMaskLognormal(v2.SPECIES{1},theMask,v.CONC_CV,0);
        
end

     % NFKB_CYCD_T NFKB_BCL2_T  NFKB_MYC_T    GROWTH_T  
     % RATE_CV CONC_CV PARTITION_CV VOL_CV IKK_CV  
     % GMSyn  GMDeg  MassGrowth MassDecay 
     % CYCDtSyn CYCDtDeg BCL2Syn BCL2tDeg 
     % MYCSyn MYCDeg mMycSyn mMycDeg L
     % IKK STIMULUS, Crel monomer,cRelFeedback,eps (Ribosome modifier);IKKt
function c2= randomCondition(maxTime,parameters)
    c2 = struct;
    c2.NFKB_CYCD_T= parameters(1);
    c2.NFKB_BCL2_T= parameters(2);
    c2.NFKB_MYC_T=  parameters(3);
    c2.GROWTH_T=    parameters(4);
    c2.RATE_CV=     parameters(5);
    c2.CONC_CV=     parameters(6);
    c2.PARTITION_CV=parameters(7);
    c2.VOL_CV=      parameters(8);
    c2.IKK_CV=      parameters(9);
    c2.GMSyn=       parameters(10);
    c2.GMDeg=       parameters(11);
    c2.MassGrowth=  parameters(12);
    c2.MassDecay=   parameters(13);
    c2.CYCDtSyn=    parameters(14);
    c2.CYCDtDeg=    parameters(15);
    c2.BCL2tSyn=    parameters(16);
    c2.BCL2tDeg=	parameters(17);
    c2.MYCSyn=      parameters(18);
    c2.MYCDeg=      parameters(19);
    c2.MYCtSyn=     parameters(20);
    c2.MYCtDeg=     parameters(21);
    c2.L=           parameters(22);
    c2.STIMULUS=    parameters(23);
    c2.CREL=        parameters(24);
    c2.cRelFeedback=parameters(25);
    c2.EPS=         parameters(26);
    c2.TOTAL_IKK   =parameters(27);
    c2.pre_RATE_CV  =parameters(28);
    c2.pre_CONC_CV = parameters(29);
    c2.maxTime=maxTime;
end


function [kinit]=getRandomRatesMaskNormal(k0,mask,CV)
    kinit = zeros(length(k0),1);
    for cc=1:length(k0)
        factor = 1;
        if mask(cc) == 1
            factor = normrnd(1,double(sqrt(CV)));
        end
        kinit(cc)=factor*k0(cc);
    end
end

function [x0aa]= getRandomSpeciesMaskLognormal(x0,mask,CV,GaussianNoise)
    x0aa = zeros(length(x0),1);
    for cc=1:length(x0)
        factor = 1;
        if GaussianNoise > 0
            factor = normrnd(1,double(GaussianNoise));
        end
        if CV >0
            sigma = sqrt(log(CV*CV+1));
        else
            sigma = 0;
        end
        mu= log(max(0,x0(cc))); 
        if mask(cc) == 1
            rnd = lognrnd(mu,sigma);
            if mu==0 
                rnd=0;
            end
            x0aa(cc)=rnd*factor;
         
        else
            x0aa(cc)=x0(cc)*factor;
        end
    end
end


